/*
 * Copyright 2010 by Kappich Systemberatung Aachen
 * 
 * This file is part of de.bsvrz.pua.prot.
 * 
 * de.bsvrz.pua.prot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.pua.prot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pua.prot.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bsvrz.pua.prot.client;

import de.bsvrz.pua.prot.util.JobInProgress;

/**
 * Listener-Klasse, die über fertigestellte Aufträge und gelöschte Protokolle informiert wird.
 *
 * @author Kappich Systemberatung
 * @version $Revision: 8953 $
 */
public interface ProtocolListenerEx{

	/**
	 * Diese Methode kann überschrieben werden und wird jedes mal aufgerufen wenn ein protokollbezogener Auftrag beendet wurde. Dies
	 * betrifft die folgenden Operationen:
	 *
	 * <ul> <li>Protokoll erstellen</li> <li>Ungelesenes Protokoll abrufen</li> <li>Gespeichertes Protokoll abrufen</li> </ul>
	 * <p>
	 * Der Aufruf dieser Methode ist unabhängig davon, ob die Erstellung/Übertragung erfolgreich war oder nicht. Details
	 * zum Status des Auftrags lassen sich dem JobInProgress-Objekt entnehmen
	 *
	 * @param job Details zum Auftrag
	 */
	public void jobFinished(JobInProgress job);

	/**
	 * Diese Methode kann überschrieben werden und wird jedes mal aufgerufen, wenn ein Protokoll gelöscht wurde. Diese Operation wird nur
	 * von neueren PuA-Server-Versionen unterstützt
	 *
	 * @param protocolId ID des gelöschten Protokolls
	 */
	public void protocolRemoved(long protocolId);

	/**
	 * Diese Methode kann überschrieben werden und wird jedes mal aufgerufen, wenn ein ungelesenes Protokoll gelesen wurde und damit zu
	 * einem gespeicherten Protokoll wird. Diese Operation wird nur von neueren PuA-Server-Versionen unterstützt
	 *
	 * @param protocolId ID des gelöschten Protokolls
	 */
	public void protocolRead(long protocolId);
}
