/*
 * Copyright 2005-2008 by beck et al. projects GmbH, Munich
 * Copyright 2009 by Kappich Systemberatung, Aachen
 * 
 * This file is part of de.bsvrz.pua.prot.
 * 
 * de.bsvrz.pua.prot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.pua.prot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pua.prot.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bsvrz.pua.prot.functions;

import antlr.collections.AST;
import de.bsvrz.pua.prot.exceptions.SemanticErrorException;
import de.bsvrz.pua.prot.processing.util.BaseDataSet;
import de.bsvrz.pua.prot.processing.util.ExpressionResultAndState;
import de.bsvrz.pua.prot.processing.util.TempAttribut;
import de.bsvrz.pua.prot.util.ErrorMessageBuilder;
import de.bsvrz.pua.prot.util.ExpressionResult;
import de.bsvrz.pua.prot.util.ExpressionResult.ResultType;
import de.bsvrz.pua.prot.util.ExpressionTree;
import de.bsvrz.pua.prot.util.attributes.RealElement;
import de.bsvrz.pua.prot.util.attributes.TempAttributeDescription;

import java.util.*;


/**
 * Bildet den Betrag.
 *
 * @author beck et al. projects GmbH
 * @author Martin Hilgers
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class Abs implements ExpressionInterface {

	public ExpressionResult evaluate(
			List<ExpressionTree> expressionList,
			Hashtable<RealElement, BaseDataSet> realElements,
			Hashtable<TempAttributeDescription, TempAttribut> tempElements, final int pseudoObjectIndex) {
		if(expressionList == null || expressionList.size() != 1) {
			return ExpressionResult.error(ErrorMessageBuilder.illegalArgumentCount(this, expressionList, false, ErrorMessageBuilder.NUMERIC));
		}
		else {
			ExpressionResult result = new ExpressionResult();

			ExpressionTree xTree = expressionList.get(0);

			ExpressionResultAndState xValue = xTree.evaluateEx(realElements, tempElements, pseudoObjectIndex);

			if(xValue.getType() == ResultType.LONG || xValue.getType() == ResultType.DOUBLE) {
				result.set(Math.abs(xValue.getResult().getDouble()));
			}
			else {
				return ExpressionResult.error(ErrorMessageBuilder.illegalArgument(this, expressionList, 0, xValue, ErrorMessageBuilder.NUMERIC));			
			}

			return result;
		}
	}

	public ResultType getResultType(final List<ExpressionTree> expressionList, final AST ast) throws SemanticErrorException {
		if(expressionList == null || expressionList.size() != 1) return ResultType.ERROR;
		ExpressionTree xTree = expressionList.get(0);
		final ResultType type = xTree.getResultType(ast);
		if(type == ResultType.DOUBLE || type == ResultType.LONG) return ResultType.DOUBLE;
		return ResultType.ERROR;
	}

	@Override
	public String getFunctionName() {
		return "abs";
	}
}
